@@ -71,12 +71,7 @@ void KFileItemTest::testPermissionsString()
                        QFile::ReadOwner | QFile::WriteOwner | QFile::ExeUser | QFile::ReadGroup | QFile::ExeGroup | QFile::ReadOther | QFile::ExeOther));
     QFile folderFile(tempDir.path() + "/afolder");
     KFileItem folderItem(QUrl::fromLocalFile(folderFile.fileName()), QString(), KFileItem::Unknown);
-#ifdef Q_OS_FREEBSD
-    // QDir::mkdir does not apply permissions correctly on FreeBSD
-    QCOMPARE(folderItem.permissionsString(), QStringLiteral("drwx------")); // 700
-#else
     QCOMPARE(folderItem.permissionsString(), QStringLiteral("drwxr-xr-x")); // 755
-#endif
     QVERIFY(folderItem.isReadable());
     QCOMPARE(folderItem.getStatusBarInfo(), CaseInsensitiveStringCompareHelper(QStringLiteral("afolder (Folder)")));
 
@@ -703,6 +698,19 @@ void KFileItemTest::testEncodeFileName()
     QCOMPARE(KIO::encodeFileName(text), expectedFileName);
 }
 
+void KFileItemTest::testSquareBracketsInFileName()
+{
+#if QT_VERSION == QT_VERSION_CHECK(6, 8, 3) || QT_VERSION == QT_VERSION_CHECK(6, 9, 0)
+    QSKIP("This test is expected to fail on Qt 6.8.3 / 6.9.0");
+#endif
+    QString dir = QStringLiteral("/tmp[%]");
+    QString file = QStringLiteral("[%].txt");
+    KIO::UDSEntry entry;
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, file);
+    KFileItem item(entry, QUrl::fromLocalFile(dir), true, true);
+    QCOMPARE(item.url(), QUrl::fromLocalFile(dir + QStringLiteral("/") + file));
+}
+
 void KFileItemTest::testListProperties_data()
 {
     QTest::addColumn<QString>("itemDescriptions");
