@@ -21,11 +21,17 @@
 #include <KPluginFactory>
 #include <KPluginMetaData>
 #include <KSandbox>
+#include <KWindowSystem>
 #include <jobuidelegatefactory.h>
 #include <kapplicationtrader.h>
 #include <kdirnotify.h>
 #include <kurlauthorized.h>
 
+#if HAVE_WAYLAND
+#include <KWaylandExtras>
+#endif
+
+#include <QApplication>
 #include <QFile>
 #include <QMenu>
 #include <QMimeDatabase>
@@ -36,6 +42,7 @@
 #include <QDBusConnectionInterface>
 #include <QDBusInterface>
 #include <QDBusMessage>
+#include <QDBusUnixFileDescriptor>
 #endif
 #include <algorithm>
 #include <kio_widgets_debug.h>
@@ -580,8 +587,13 @@ int KFileItemActionsPrivate::addPluginActionsTo(QMenu *mainMenu, QMenu *actionsM
 
         KAbstractFileItemActionPlugin *abstractPlugin = m_loadedPlugins.value(pluginId);
         if (!abstractPlugin) {
-            abstractPlugin = KPluginFactory::instantiatePlugin<KAbstractFileItemActionPlugin>(jsonMetadata, this).plugin;
-            m_loadedPlugins.insert(pluginId, abstractPlugin);
+            auto result = KPluginFactory::instantiatePlugin<KAbstractFileItemActionPlugin>(jsonMetadata, this);
+            if (result) {
+                abstractPlugin = result.plugin;
+                m_loadedPlugins.insert(pluginId, abstractPlugin);
+            } else {
+                qCWarning(KIO_WIDGETS) << "Couldn't load plugin:" << pluginId << result.errorText << result.errorReason;
+            }
         }
         if (abstractPlugin) {
             connect(abstractPlugin, &KAbstractFileItemActionPlugin::error, q, &KFileItemActions::error);
@@ -699,17 +711,54 @@ void KFileItemActionsPrivate::insertOpenWithActionsTo(QAction *before, QMenu *to
 #ifdef WITH_QTDBUS
     if (KSandbox::isInside() && !m_fileOpenList.isEmpty()) {
         auto openWithAction = makeOpenWithAction();
-        QObject::connect(openWithAction, &QAction::triggered, this, [this] {
+
+        auto sendMessage = [this](const QString &token) {
             const auto &items = m_fileOpenList;
             for (const auto &fileItem : items) {
+                auto local = fileItem.isLocalFile();
                 QDBusMessage message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.portal.Desktop"),
                                                                       QLatin1String("/org/freedesktop/portal/desktop"),
                                                                       QLatin1String("org.freedesktop.portal.OpenURI"),
-                                                                      QLatin1String("OpenURI"));
-                message << QString() << fileItem.url() << QVariantMap{};
+                                                                      local ? QLatin1String("OpenFile") : QLatin1String("OpenURI"));
+                QVariant uriOrFd;
+                if (local) {
+                    QFile localFile(fileItem.localPath());
+                    if (!localFile.open(QIODevice::ReadOnly))
+                        continue;
+
+                    uriOrFd = QVariant::fromValue(QDBusUnixFileDescriptor(localFile.handle()));
+                } else {
+                    uriOrFd = fileItem.url();
+                }
+                message << QString() << uriOrFd;
+                message << (token.isNull() ? QVariantMap{} : QVariantMap{{QStringLiteral("activation_token"), token}});
                 QDBusConnection::sessionBus().asyncCall(message);
             }
+        };
+#if HAVE_WAYLAND
+        if (KWindowSystem::isPlatformWayland() && !qGuiApp->allWindows().isEmpty()) {
+            QObject::connect(openWithAction, &QAction::triggered, this, [] {
+                auto window = qGuiApp->allWindows().constFirst();
+                KWaylandExtras::requestXdgActivationToken(window, KWaylandExtras::lastInputSerial(window), {});
+            });
+            QObject::connect(
+                KWaylandExtras::self(),
+                &KWaylandExtras::xdgActivationTokenArrived,
+                this,
+                [sendMessage](int, const QString &token) {
+                    sendMessage(token);
+                },
+                Qt::SingleShotConnection);
+        } else {
+            QObject::connect(openWithAction, &QAction::triggered, this, [sendMessage] {
+                sendMessage({});
+            });
+        }
+#else
+        QObject::connect(openWithAction, &QAction::triggered, this, [sendMessage] {
+            sendMessage({});
         });
+#endif
         topMenu->insertAction(before, openWithAction);
         return;
     }
@@ -758,6 +807,15 @@ void KFileItemActionsPrivate::insertOpenWithActionsTo(QAction *before, QMenu *to
             topMenu->insertAction(before, act);
         }
 
+        // Only now remove NoDisplay services. They should work as default application
+        // but not offered as an alternative in the Open With submenu.
+        offers.erase(std::remove_if(offers.begin(),
+                                    offers.end(),
+                                    [](KService::Ptr service) {
+                                        return service->noDisplay();
+                                    }),
+                     offers.end());
+
         // If there are still more apps, show them in a sub-menu
         if (!offers.isEmpty()) { // submenu 'open with'
             QMenu *subMenu = new QMenu(isDir ? i18nc("@title:menu", "&Open Folder With") : i18nc("@title:menu", "&Open With"), topMenu);
