@@ -101,6 +101,7 @@ public:
     void gotNewIcon(const QModelIndex &index);
 
     void paintJobTransfers(QPainter *painter, const qreal &jobAnimationAngle, const QPoint &iconPos, const QStyleOptionViewItem &opt);
+    int scaledEmblemSize(int iconSize) const;
 
 public:
     KFileItemDelegate::InformationList informationList;
@@ -112,6 +113,7 @@ public:
     QTextOption::WrapMode wrapMode;
     bool jobTransfersVisible;
     QIcon downArrowIcon;
+    QRect emblemRect;
 
 private:
     KIO::DelegateAnimationHandler *animationHandler;
@@ -128,6 +130,7 @@ KFileItemDelegate::Private::Private(KFileItemDelegate *parent)
     , showToolTipWhenElided(true)
     , wrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere)
     , jobTransfersVisible(false)
+    , emblemRect(QRect())
     , animationHandler(new KIO::DelegateAnimationHandler(parent))
     , activeMargins(nullptr)
 {
@@ -824,7 +827,7 @@ KFileItemDelegate::KFileItemDelegate(QObject *parent)
     d->setHorizontalMargin(Private::ItemMargin, 0, 0);
 
     // Margins for vertical mode (icon views)
-    d->setVerticalMargin(Private::TextMargin, 6, 2);
+    d->setVerticalMargin(Private::TextMargin, 6, 0);
     d->setVerticalMargin(Private::IconMargin, focusHMargin, focusVMargin);
     d->setVerticalMargin(Private::ItemMargin, 0, 0);
 
@@ -1132,6 +1135,7 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
     }
 
     const QAbstractItemView *view = qobject_cast<const QAbstractItemView *>(opt.widget);
+    const qreal dpr = painter->device()->devicePixelRatioF();
 
     // Check if the item is being animated
     // ========================================================================
@@ -1150,7 +1154,7 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
     }
 
     QIcon::State iconState = option.state & QStyle::State_Open ? QIcon::On : QIcon::Off;
-    QPixmap icon = opt.icon.pixmap(opt.decorationSize, iconMode, iconState);
+    QPixmap icon = opt.icon.pixmap(opt.decorationSize, dpr, iconMode, iconState);
 
     const KFileItem fileItem = d->fileItem(index);
     if (fileItem.isHidden()) {
@@ -1176,6 +1180,7 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
                     pixmap = d->transition(fadeFromPixmap, pixmap, state->fadeProgress());
                 }
                 painter->drawPixmap(option.rect.topLeft(), pixmap);
+                drawSelectionEmblem(option, painter, index);
                 if (d->jobTransfersVisible && index.column() == 0) {
                     if (index.data(KDirModel::HasJobRole).toBool()) {
                         d->paintJobTransfers(painter, state->jobAnimationAngle(), iconPos, opt);
@@ -1236,21 +1241,33 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
     int focusVMargin = style->pixelMetric(QStyle::PM_FocusFrameVMargin);
     QRect focusRect = textBoundingRect.adjusted(-focusHMargin, -focusVMargin, +focusHMargin, +focusVMargin);
 
+    auto drawBackground = [textBoundingRect, iconPos, style](QPainter &painter, QStyleOptionViewItem &opt) {
+        if (opt.decorationPosition != QStyleOptionViewItem::Position::Top) {
+            style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, &painter, opt.widget);
+        } else {
+            auto oldrect = opt.rect;
+            opt.rect = textBoundingRect;
+            style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, &painter, opt.widget);
+            opt.rect = QRect(iconPos, opt.decorationSize);
+            style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, &painter, opt.widget);
+            opt.rect = oldrect;
+        }
+    };
+
     // Create a new cached rendering of a hovered and an unhovered item.
     // We don't create a new cache for a fully hovered item, since we don't
     // know yet if a hover out animation will be run.
     // ========================================================================
     if (state && (state->hoverProgress() < 1 || state->fadeProgress() < 1)) {
-        const qreal dpr = painter->device()->devicePixelRatioF();
-
         cache = new KIO::CachedRendering(opt.state, option.rect.size(), index, dpr);
 
         QPainter p;
         p.begin(&cache->regular);
         p.translate(-option.rect.topLeft());
         p.setRenderHint(QPainter::Antialiasing);
-        style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, &p, opt.widget);
+        drawBackground(p, opt);
         p.drawPixmap(iconPos, icon);
+        drawSelectionEmblem(option, painter, index);
         d->drawTextItems(&p, labelLayout, labelColor, infoLayout, infoColor, textBoundingRect);
         d->drawFocusRect(&p, opt, focusRect);
         p.end();
@@ -1261,8 +1278,9 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
         p.begin(&cache->hover);
         p.translate(-option.rect.topLeft());
         p.setRenderHint(QPainter::Antialiasing);
-        style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, &p, opt.widget);
+        drawBackground(p, opt);
         p.drawPixmap(iconPos, icon);
+        drawSelectionEmblem(option, painter, index);
         d->drawTextItems(&p, labelLayout, labelColor, infoLayout, infoColor, textBoundingRect);
         d->drawFocusRect(&p, opt, focusRect);
         p.end();
@@ -1282,6 +1300,7 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
         }
 
         painter->drawPixmap(option.rect.topLeft(), pixmap);
+        drawSelectionEmblem(option, painter, index);
         painter->setRenderHint(QPainter::Antialiasing);
         if (d->jobTransfersVisible && index.column() == 0) {
             if (index.data(KDirModel::HasJobRole).toBool()) {
@@ -1301,11 +1320,12 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
         icon = d->applyHoverEffect(icon);
     }
 
-    style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, painter, opt.widget);
+    drawBackground(*painter, opt);
     painter->drawPixmap(iconPos, icon);
 
     d->drawTextItems(painter, labelLayout, labelColor, infoLayout, infoColor, textBoundingRect);
     d->drawFocusRect(painter, opt, focusRect);
+    drawSelectionEmblem(option, painter, index);
 
     if (d->jobTransfersVisible && index.column() == 0 && state) {
         if (index.data(KDirModel::HasJobRole).toBool()) {
@@ -1315,6 +1335,33 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
     painter->restore();
 }
 
+void KFileItemDelegate::drawSelectionEmblem(QStyleOptionViewItem option, QPainter *painter, const QModelIndex &index) const
+{
+    if (index.column() != 0 || !qApp->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick)) {
+        return;
+    }
+    const auto state = option.state;
+    if ((state & QStyle::State_MouseOver && !fileItem(index).isDir()) || (state & QStyle::State_Selected)) {
+        const QString selectionEmblem = state & QStyle::State_Selected ? QStringLiteral("emblem-remove") : QStringLiteral("emblem-added");
+        const auto emblem = QIcon::fromTheme(selectionEmblem).pixmap(d->emblemRect.size(), state & QStyle::State_MouseOver ? QIcon::Active : QIcon::Disabled);
+
+        painter->drawPixmap(d->emblemRect.topLeft(), emblem);
+    }
+}
+
+int KFileItemDelegate::Private::scaledEmblemSize(int iconSize) const
+{
+    if (iconSize <= KIconLoader::SizeSmallMedium) {
+        return KIconLoader::SizeSmall;
+    } else if (iconSize <= KIconLoader::SizeHuge) {
+        return KIconLoader::SizeSmallMedium;
+    } else if (iconSize <= KIconLoader::SizeEnormous) {
+        return KIconLoader::SizeMedium;
+    }
+
+    return KIconLoader::SizeHuge;
+}
+
 QWidget *KFileItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
     QStyleOptionViewItem opt(option);
@@ -1551,4 +1598,26 @@ bool KFileItemDelegate::eventFilter(QObject *object, QEvent *event)
     } // switch (event->type())
 }
 
+void KFileItemDelegate::setSelectionEmblemRect(QRect rect, int iconSize)
+{
+    const auto emblemSize = d->scaledEmblemSize(iconSize);
+
+    // With small icons, try to center the emblem on top of the icon
+    if (iconSize <= KIconLoader::SizeSmallMedium) {
+        d->emblemRect = QRect(rect.topLeft().x() + emblemSize / 4, rect.topLeft().y() + emblemSize / 4, emblemSize, emblemSize);
+    } else {
+        d->emblemRect = QRect(rect.topLeft().x(), rect.topLeft().y(), emblemSize, emblemSize);
+    }
+}
+
+QRect KFileItemDelegate::selectionEmblemRect() const
+{
+    return d->emblemRect;
+}
+
+KFileItem KFileItemDelegate::fileItem(const QModelIndex &index) const
+{
+    return d->fileItem(index);
+}
+
 #include "moc_kfileitemdelegate.cpp"
